==> fryhcs-0.2.0.tar.gz: fryhcs-0.2.0/src/fryhcs/page.py <==
from fryhcs.element import Element, component_id_attr_name, children_attr_name, type_attr_name
from fryhcs.utils import static_url
from fryhcs.config import fryconfig
from importlib import import_module
import json

class Page(object):
    def __init__(self):
        # 记录当前已经处理的组件script元素列表，列表长度是当前正在处理的组件的ID
        self.components = []
        # 组件实例ID到子组件引用/引用列表的映射关系, cid -> (refname -> childcid | list of childcid)
        self.cid2childrefs = {}
        self.hasjs = False

    def add_component(self, component):
        self.components.append(component)
        cid = len(self.components)
        component['fryid'] = cid
        self.cid2childrefs[cid] = {} 
        return cid

    def add_ref(self, cid, refname, childcid):
        childrefs = self.cid2childrefs[cid]
        if refname in childrefs:
            refs = childrefs[refname]
        else:
            refs = set()
            childrefs[refname] = refs
        refs.add(childcid)

    def child_refs(self, cid):
        origin = self.cid2childrefs.get(cid, {})
        refs = {}
        for name, ids in origin.items():
            if name.endswith(':a'):
                refs[name[:-2]] = list(ids)
            else:
                if len(ids) != 1:
                    raise RuntimeError(f"More than ONE ref value for '{name}'.")
                refs[name] = ids.pop()
        return refs


def render(element, props={}):
    page = Page()
    if isinstance(element, Element):
        element = element.render(page)
    elif callable(element) and getattr(element, '__name__', 'anonym')[0].isupper():
        element = Element(element, props).render(page)
    elif isinstance(element, str):
        if '.' in element:
            module, _, comp = element.rpartition('.')
            module = import_module(module)
            component = getattr(module, comp)
            if callable(component) and comp[0].isupper():
                element = Element(component, props).render(page)
        elif element and element[0].islower():
            element = Element(element, props).render(page)
    return element


def html(content='div', title='', lang='en', rootclass='', charset='utf-8', viewport="width=device-width, initial-scale=1.0", metas={}, properties={}, equivs={}):
    sep = '\n    '
    main_content = render(content)
    page = main_content.page
    components = []
    for c in page.components:
        content = {
            'name': c['fryname'],
            'refs': c['fryrefs'],
            'args': c['fryargs'],
            'url':  c['fryurl'],
        }
        scriptprops = {
            type_attr_name: 'text/x-frydata',
            children_attr_name: [json.dumps(content)],
            component_id_attr_name: c['fryid'],
        }
        comp = Element('script', scriptprops, True)
        components.append(str(comp))
    components = '\n    '.join(components)
    if not page.hasjs:
        hydrate_script = ""
    else:
        hydrate_script = f"""
    <script type="module">
      let components = {{}};
      const scripts = document.querySelectorAll('script[data-fryid]');
      let firsturl = undefined;
      for (const script of scripts) {{
        const cid = script.dataset.fryid;
        script.fryid = cid;
        const data = JSON.parse(script.textContent);
        script.fryname = data.name;
        script.fryurl  = data.url;
        if (typeof firsturl === 'undefined') {{
            firsturl = data.url;
        }}
        script.fryargs = Object.assign({{}}, data.args);
        script.fryrefs = Object.assign({{}}, data.refs);
        components[cid] = script;
      }}
      const {{ hydrateAll }} = await import(firsturl);
      await hydrateAll(document.body, components);
    </script>
"""

    metas = sep.join(f'<meta name="{name}" content="{value}">'
                       for name, value in metas.items())
    properties = sep.join(f'<meta property="{property}" content="{value}">'
                            for property, value in properties.items())
    equivs = sep.join(f'<meta http-equiv="{equiv}" content="{value}">'
                            for equiv, value in equivs.items())
    # no need to use importmap
    #importmap = f'''
    #<script type="importmap">
    #  {{
    #    "imports": {{
    #      "fryhcs": "{static_url('js/fryhcs.js')}"
    #    }}
    #  }}
    #</script>
    #'''

    if fryconfig.debug:
        script = """
    <script type="module">
      let serverId = null;
      let eventSource = null;
      let timeoutId = null;
      function checkAutoReload() {
          if (timeoutId !== null) clearTimeout(timeoutId);
          timeoutId = setTimeout(checkAutoReload, 1000);
          if (eventSource !== null) eventSource.close();
          eventSource = new EventSource("{{autoReloadPath}}");
          eventSource.addEventListener('open', () => {
              console.log(new Date(), "Auto reload connected.");
              if (timeoutId !== null) clearTimeout(timeoutId);
              timeoutId = setTimeout(checkAutoReload, 1000);
          });
          eventSource.addEventListener('message', (event) => {
              const data = JSON.parse(event.data);
              if (serverId === null) {
                  serverId = data.serverId;
              } else if (serverId !== data.serverId) {
                  if (eventSource !== null) eventSource.close();
                  if (timeoutId !== null) clearTimeout(timeoutId);
                  location.reload();
                  return;
              }
              if (timeoutId !== null) clearTimeout(timeoutId);
              timeoutId = setTimeout(checkAutoReload, 1000);
          });
      }
      checkAutoReload();
    </script>
"""
        autoreload = script.replace('{{autoReloadPath}}', fryconfig.check_reload_url)
    else:
        autoreload = ''

    if rootclass:
        rootclass = f' class="{rootclass}"'
    else:
        rootclass = ''

    return f'''\
<!DOCTYPE html>
<html lang={lang}{rootclass}>
  <head>
    <meta charset="{charset}">
    <title>{title}</title>
    <meta name="viewport" content="{viewport}">
    {metas}
    {properties}
    {equivs}
    <link rel="stylesheet" href="{static_url(fryconfig.css_url)}">
  </head>
  <body>
    {main_content}
    <div style="display:none;">
    {components}
    </div>
    {hydrate_script}
    {autoreload}
  </body>
</html>
'''

==> fryhcs-0.2.4.tar.gz: fryhcs-0.2.4/src/fryhcs/page.py <==
from fryhcs.element import Element, component_id_attr_name, children_attr_name, type_attr_name
from fryhcs.utils import static_url
from fryhcs.config import fryconfig
from importlib import import_module
import json

class Page(object):
    def __init__(self):
        # 记录当前已经处理的组件script元素列表，列表长度是当前正在处理的组件的ID
        self.components = []
        # 组件实例ID到子组件引用/引用列表的映射关系, cid -> (refname -> childcid | list of childcid)
        self.cid2childrefs = {}
        self.hasjs = False

    def add_component(self, component):
        self.components.append(component)
        cid = len(self.components)
        component['fryid'] = cid
        self.cid2childrefs[cid] = {} 
        return cid

    def add_ref(self, cid, refname, childcid):
        childrefs = self.cid2childrefs[cid]
        if refname in childrefs:
            refs = childrefs[refname]
        else:
            refs = set()
            childrefs[refname] = refs
        refs.add(childcid)

    def child_refs(self, cid):
        origin = self.cid2childrefs.get(cid, {})
        refs = {}
        for name, ids in origin.items():
            if name.endswith(':a'):
                refs[name[:-2]] = list(ids)
            else:
                if len(ids) != 1:
                    raise RuntimeError(f"More than ONE ref value for '{name}'.")
                refs[name] = ids.pop()
        return refs


def render(element, **kwargs):
    page = Page()
    if isinstance(element, Element):
        element = element.render(page)
    elif callable(element) and getattr(element, '__name__', 'anonym')[0].isupper():
        element = Element(element, kwargs).render(page)
    elif isinstance(element, str):
        if '.' in element:
            module, _, comp = element.rpartition('.')
            module = import_module(module)
            component = getattr(module, comp)
            if callable(component) and comp[0].isupper():
                element = Element(component, kwargs).render(page)
        elif element and element[0].islower():
            element = Element(element, kwargs).render(page)
    return element


def html(content='div', title='', lang='en', rootclass='', charset='utf-8', viewport="width=device-width, initial-scale=1.0", metas={}, properties={}, equivs={}):
    sep = '\n    '
    main_content = render(content)
    page = main_content.page
    components = []
    for c in page.components:
        content = {
            'name': c['fryname'],
            'refs': c['fryrefs'],
            'args': c['fryargs'],
            'url':  c['fryurl'],
        }
        scriptprops = {
            type_attr_name: 'text/x-frydata',
            children_attr_name: [json.dumps(content)],
            component_id_attr_name: c['fryid'],
        }
        comp = Element('script', scriptprops, True)
        components.append(str(comp))
    components = '\n    '.join(components)
    if not page.hasjs:
        hydrate_script = ""
    else:
        hydrate_script = f"""
    <script type="module">
      let components = {{}};
      const scripts = document.querySelectorAll('script[data-fryid]');
      let firsturl = undefined;
      for (const script of scripts) {{
        const cid = script.dataset.fryid;
        script.fryid = cid;
        const data = JSON.parse(script.textContent);
        script.fryname = data.name;
        script.fryurl  = data.url;
        if (typeof firsturl === 'undefined') {{
            firsturl = data.url;
        }}
        script.fryargs = Object.assign({{}}, data.args);
        script.fryrefs = Object.assign({{}}, data.refs);
        components[cid] = script;
      }}
      const {{ hydrateAll }} = await import(firsturl);
      await hydrateAll(document.body, components);
    </script>
"""

    metas = sep.join(f'<meta name="{name}" content="{value}">'
                       for name, value in metas.items())
    properties = sep.join(f'<meta property="{property}" content="{value}">'
                            for property, value in properties.items())
    equivs = sep.join(f'<meta http-equiv="{equiv}" content="{value}">'
                            for equiv, value in equivs.items())
    # no need to use importmap
    #importmap = f'''
    #<script type="importmap">
    #  {{
    #    "imports": {{
    #      "fryhcs": "{static_url('js/fryhcs.js')}"
    #    }}
    #  }}
    #</script>
    #'''

    if fryconfig.debug:
        script = """
    <script type="module">
      let serverId = null;
      let eventSource = null;
      let timeoutId = null;
      function checkAutoReload() {
          if (timeoutId !== null) clearTimeout(timeoutId);
          timeoutId = setTimeout(checkAutoReload, 1000);
          if (eventSource !== null) eventSource.close();
          eventSource = new EventSource("{{autoReloadPath}}");
          eventSource.addEventListener('open', () => {
              console.log(new Date(), "Auto reload connected.");
              if (timeoutId !== null) clearTimeout(timeoutId);
              timeoutId = setTimeout(checkAutoReload, 1000);
          });
          eventSource.addEventListener('message', (event) => {
              const data = JSON.parse(event.data);
              if (serverId === null) {
                  serverId = data.serverId;
              } else if (serverId !== data.serverId) {
                  if (eventSource !== null) eventSource.close();
                  if (timeoutId !== null) clearTimeout(timeoutId);
                  location.reload();
                  return;
              }
              if (timeoutId !== null) clearTimeout(timeoutId);
              timeoutId = setTimeout(checkAutoReload, 1000);
          });
      }
      checkAutoReload();
    </script>
"""
        autoreload = script.replace('{{autoReloadPath}}', fryconfig.check_reload_url)
    else:
        autoreload = ''

    if rootclass:
        rootclass = f' class="{rootclass}"'
    else:
        rootclass = ''

    return f'''\
<!DOCTYPE html>
<html lang={lang}{rootclass}>
  <head>
    <meta charset="{charset}">
    <title>{title}</title>
    <meta name="viewport" content="{viewport}">
    {metas}
    {properties}
    {equivs}
    <link rel="stylesheet" href="{static_url(fryconfig.css_url)}">
  </head>
  <body>
    {main_content}
    <div style="display:none;">
    {components}
    </div>
    {hydrate_script}
    {autoreload}
  </body>
</html>
'''
